require 'json'
require 'net/http'
require 'open-uri'

module Appscreens
  ITUNES_SEARCH_ENDPOINT = "https://itunes.apple.com/"
  ITUNES_SEARCH_RESULTS_KEY = "results"
  APP_SCREENSHOTS_KEY = "screenshotUrls"
  APP_SCREENSHOTS_KEY_IPAD = "ipadScreenshotUrls"
  APP_NAME_KEY = "trackName"
  APP_ICON_KEY = "artworkUrl100"

  class Client
    def initialize
      @search_url = ITUNES_SEARCH_ENDPOINT
      @country = 'US'
    end

    def lookup(app_id)
      uri = URI.join(@search_url, "lookup")
      uri.query = URI.encode_www_form( {id: app_id} )

      json_response_from_uri(uri)[ITUNES_SEARCH_RESULTS_KEY].first
    end

    def download_screenshots(app:, icon:, ipad:, destination:)
      screenshots = app[ipad ? APP_SCREENSHOTS_KEY_IPAD : APP_SCREENSHOTS_KEY]
      app_name = app[APP_NAME_KEY].downcase.tr(" ","_")

      screenshots.each_with_index do |screen_url, index|
        screen_url = screen_url.sub('480x480', '960x960') if ipad
        puts "Downloading screenshot #{index + 1} of #{screenshots.length}..."
        file_name = "#{destination}/#{app_name}_#{index + 1}#{ipad ? "_ipad" : ""}.jpeg"
        File.open(file_name, 'wb') { |fp| fp.write(open(screen_url).read) }
      end

      if icon
        puts "Downloading app icon..."
        File.open("#{destination}/#{app_name}_icon.png", 'wb') { |fp| fp.write(open(app[APP_ICON_KEY]).read) }
      end

      puts '#######################################################################'.green
      puts "Finished grabbing artwork for #{app[APP_NAME_KEY]}".green
      puts '#######################################################################'.green

    end

    private

    def json_response_from_uri(uri)
      response = Net::HTTP.get_response(uri)
      JSON.parse(response.body) rescue nil
    end

  end
end